/*! @file
 * @brief
 * small program to evaluate integer expressions
 */


#include "at_startup.H"

// sanity checks
#ifdef IS_CLIENT
 #error "ieval is not a client!"
#endif

#ifdef IS_SERVER
 #error "ieval is not a server!"
#endif

#ifdef USE_NETWORK
 #error "ieval uses no network features!"
#endif

#ifndef IS_STANDALONE
 #error "ieval is a standalone application!"
#endif



#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <gmp.h>


#ifdef NOTIFY_PARENT
 #warning "undefining NOTIFY_PARENT, since this is not a networked server"
 #undef NOTIFY_PARENT
#endif


#undef VERBOSE_WARN
#undef VERBOSE_INFO
#undef VERBOSE_NOTICE
#include "parse_term.cc"

int main(const int argc, const char* const argv[])
{
  bool showhelp = (argc!=2);

  if (argc==2)
   {
     const string s = argv[1];
     showhelp = (s=="-h" || s=="--help");
   }
   
  if (showhelp)
    {
      PrintHeader("ieval expression evaluator");
      cout << "ieval is a small tool to evaluate integer expressions." << endl
           << "Terms consisting of 0123456789 as decimal digits," << endl
           << " +,-,*,^,/,% and :,** as operators, () for parenthesis," << endl
           << "and the function symbols" << endl
           << " F [Fibonacci number], L [Lucas number], P [partition number]," << endl
           << " f [factorial], i [increment], d [decrement]," << endl
           << " p [next prime number (if not already prime)]," << endl
           << " r [(truncated) square root and" << endl
           << " s [next safeprime (if not already safeprime)]" << endl
           << "are allowed. The result is printed to stdout." << endl
           << "Examples: \"i10*d10\", \"P100\", \"F9*L9\", \"3^3^3\"" << endl;   
      if (argc<3) return 0; else return 1;
    }
  
  mpz_t n; // evaluated result of the given term
  mpz_init(n);

  // get and evaluate given number
  char* const neuer_str = strdup(argv[1]);
  char* str = neuer_str; 
  if (!parse_term::get_number(n, str))
   {
     cerr << "integer term for evaluation expected!" << endl;
     cerr << "Wrong input at: '" << str << "'" << endl;
     exit(1);
   }
  else
   if (str[0]!='\0')
    {
      cerr << "integer term for evaluation expected!" << endl;    
      cerr << "Syntax error in input term. (parenthesis?)" << endl;
      exit(1);
    }

  //cout << argv[1] << endl;
  //cout << neuer_str << endl;
  mpz_out_str(NULL,10,n);
  cout << endl;

  free(neuer_str); // don't call "delete []" because of "stdup/malloc"
  mpz_clear(n);

  return 0;
}
